// Sample CPP Net Kratos 3_0 Project.cpp : main project file.

#include "stdafx.h"
using namespace Kratos_3;
using namespace Kratos_3::Network;
using namespace Kratos_3::RunTimePlatform;

using namespace System;

void printOutFactletResults(DataFrame^ df, FactSetOnDemand^ fsod) {
	if(df->hasError()) {
		df->throwErrorIfAnyIsPresent();
	}
	Console::WriteLine(fsod->GetEventLogger()->getOutputBuffer());
	for(int row = 0; row < df->getNumberOfRows(); row++) {
		for(int col = 0; col < df->getNumberOfColumns(); col++) {
			Object^ o = df->getCellAt(row, col); 
			Console::WriteLine("Row: " + row + ", Col: " + col + ", Value: " + o->ToString());
		}
	}
}

int main(array<System::String ^> ^args)
{
	FactSetOnDemand^ fsod = gcnew FactSetOnDemand();
	DataFrame^ df;


    // Set our configuration. We definitely need to set our username and password, as given to us by FactSet Support staff
    IConfigManager^ config = fsod->getConfig();    /// 
    //config->setConfig(ConfigOptions::DataDirectUserName, <INSERTUSERNAMEHERE>);
    //config->setConfig(ConfigOptions::DataDirectPassword, <INSERTPWHERE>);

	array<String^>^ dates = gcnew array<String^>{ "04/10/2013", "04/11/2013", "04/12/2013" };
	array<double>^ fds_price = gcnew array<double>{ 93.33, Double::NaN, 92.65};
	array<double>^ fds_volume = gcnew array<double> { 388.205, Double::NaN, 359.396 };
	array<double>^ xom_price = gcnew array<double> { 88.68, Double::NaN, 359.396 };
	array<double>^ xom_volume = gcnew array<double> { 14826.0, Double::NaN, 14980.0 };

    UploadFactlet^ uFactlet = gcnew UploadFactlet("TestOFDB_CSharp", gcnew array<String^>{"ID","Date","my_price","my_volume"}, gcnew array<String^>{"Id","Date","Double","Double"});
    for (int i = 0; i < dates->Length; i++)
    {
        uFactlet->addDataPoint("FDS");
        uFactlet->addDataPoint(dates[i]);
        uFactlet->addDataPoint(fds_price[i]);
        uFactlet->addDataPoint(fds_volume[i]);
    }
    for (int i = 0; i < dates->Length; i++)
    {
        uFactlet->addDataPoint("XOM");
        uFactlet->addDataPoint(dates[i]);
        uFactlet->addDataPoint(xom_price[i]);
        uFactlet->addDataPoint(xom_volume[i]);
    }
    df = fsod->executeFactlet(uFactlet);
    Console::WriteLine(fsod->GetEventLogAsString());
            
    df->throwErrorIfAnyIsPresent();

    // In this example, we will run ExtractFormulaHistory and examine the unique
    // IDs and Dates in the returned data
    df = fsod->ExtractFormulaHistory("xom,stl-no,fds,ibm,efc,appl,goog", "fg_eps(0,-1m,d)", "");
    printOutFactletResults(df, fsod);

    // Extract Benchmark Detail
    df = fsod->ExtractBenchmarkDetail("SP50", "items", "FG_COMPANY_NAME,FG_MKT_VALUE");
    printOutFactletResults(df, fsod);

    df = fsod->ExtractBenchmarkDetail("SP50", "items", "p_price,proper_name", gcnew array<String^> { "cutoff", "10" });
    printOutFactletResults(df, fsod);

    df = fsod->ExtractBenchmarkDetail("SP50", "items", "p_price", gcnew array<String^> { "dates", "20110115:20110120:d" });
    printOutFactletResults(df, fsod);
            
    Console::Write("Press enter to continue");
    String^ name = Console::ReadLine();
}
