﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Kratos_3;
using Kratos_3.Network;
using Kratos_3.RunTimePlatform;

namespace Sample_Net_Kratos_3_0_Project
{
    class Program
    {
        static FactSetOnDemand fsod;

        static void printOutFactletResults(DataFrame df)
        {
            if (df.hasError())
            {
                df.throwErrorIfAnyIsPresent();
            }
            Console.WriteLine(fsod.GetEventLogger().getOutputBuffer()); //Print a summary string
            Type[] types = df.getDotNetColumnTypes();
            for (int row = 0; row < df.getNumberOfRows(); row++)
            {
                for (int col = 0; col < df.getNumberOfColumns(); col++)
                {
                    Object o = df.getCellAt(row, col); // Note that we know what the type of this cell is, so we could cast it or treat it as its proper type if we needed to
                    Console.WriteLine("Row: " + row + ", Col: " + col + ", Value: " + o.ToString());
                }
            }
        }

        static void Main(string[] args)
        {
            fsod = new FactSetOnDemand();
            DataFrame df;

            // Set our configuration. We definitely need to set our username and password, as given to us by FactSet Support staff
            IConfigManager config = fsod.getConfig();    /// 
            config.setConfig(ConfigOptions.DataDirectUserName, <INSERTUSERNAMEHERE>);
            config.setConfig(ConfigOptions.DataDirectPassword, <INSERTPWHERE>);

            UploadFactlet uFactlet = new UploadFactlet("TestOFDB_CSharp", new string[]{"ID","Date","my_price","my_volume"}, new string[]{"Id","Date","Double","Double"});
            string[] dates = { "04/10/2013", "04/11/2013", "04/12/2013" };
            double[] fds_price = { 93.33, Double.NaN, 92.65 };
            double[] fds_volume = { 388.205, Double.NaN, 359.396 };
            double[] xom_price = { 88.68, Double.NaN, 89.22 };
            double[] xom_volume = { 14826.0, Double.NaN, 14980.0 };
            for (int i = 0; i < dates.Length; i++)
            {
                uFactlet.addDataPoint("FDS");
                uFactlet.addDataPoint(dates[i]);
                uFactlet.addDataPoint(fds_price[i]);
                uFactlet.addDataPoint(fds_volume[i]);
            }
            for (int i = 0; i < dates.Length; i++)
            {
                uFactlet.addDataPoint("XOM");
                uFactlet.addDataPoint(dates[i]);
                uFactlet.addDataPoint(xom_price[i]);
                uFactlet.addDataPoint(xom_volume[i]);
            }
            df = fsod.executeFactlet(uFactlet);
            df.throwErrorIfAnyIsPresent();

            // In this example, we will run ExtractFormulaHistory and examine the unique
            // IDs and Dates in the returned data
            df = fsod.ExtractFormulaHistory("xom,stl-no,fds,ibm,efc,appl,goog", "fg_eps(0,-1m,d)", "");
            printOutFactletResults(df);

            // Extract Benchmark Detail
            df = fsod.ExtractBenchmarkDetail("SP50", "items", "FG_COMPANY_NAME,FG_MKT_VALUE");
            printOutFactletResults(df);

            df = fsod.ExtractBenchmarkDetail("SP50", "items", "p_price,proper_name", new string[] { "cutoff", "10" });
            printOutFactletResults(df);

            df = fsod.ExtractBenchmarkDetail("SP50", "items", "p_price", new string[] { "dates", "20110115:20110120:d" });
            printOutFactletResults(df);

            Console.Write("Press enter to continue");
            var name = Console.ReadLine();
        }
    }
}
